@@ -88,7 +88,10 @@ dat2str_boundary({{Y, Mo, D}, {H, M, S}}) ->
     lists:flatten(
       io_lib:format("~s_~2.2.0w_~s_~w_~2.2.0w:~2.2.0w:~2.2.0w_~w",
                     [weekday(Y,Mo,D), D, int_to_mt(Mo),
-                     Y,H,M,S,random:uniform(5000)])).
+                     Y,H,M,S,bin2int(crypto:rand_bytes(4))])).
+
+bin2int(Bin) ->
+    lists:foldl(fun(N, Acc) -> Acc * 256 + N end, 0, binary_to_list(Bin)).
 
 
 smtp_init(Server, From, Recipients) ->
@@ -256,7 +259,7 @@ e(X) when X >= 26, X < 52 -> X + $a - 26;
 e(X) when X >= 52, X < 62 -> X + $0 - 52;
 e(62) -> $+;
 e(63) -> $/;
-e(X) -> erlang:fault({badchar,X}).
+e(X) -> erlang:error({badchar,X}).
 
 
 %%
