@@ -21,7 +21,7 @@
 
 %%% TODO: Implement FastCGI filter role.
 
--export([cgi_worker/7, fcgi_worker/5]).
+-export([cgi_worker/7, fcgi_worker/6]).
 
 %%%=====================================================================
 %%% Code shared between CGI and FastCGI
@@ -533,8 +533,16 @@ cgi_start_worker(Arg, Exefilename, Scriptfilename, Pathinfo, ExtraEnv, SC) ->
 
 
 
-cgi_worker(Parent, Arg, Exefilename, Scriptfilename, Pathinfo, ExtraEnv,SC) ->
-    Env = build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv,SC),
+cgi_worker(Parent, Arg, Exefilename, Scriptfilename, Pathinfo, ExtraEnv0, SC) ->
+    ExtraEnv = lists:map(fun({K,V}) when is_binary(K), is_binary(V) ->
+                                 {binary_to_list(K), binary_to_list(V)};
+                            ({K,V}) when is_binary(K) ->
+                                 {binary_to_list(K), V};
+                            ({K,V}) when is_binary(V) ->
+                                 {K, binary_to_list(V)};
+                            (KV) -> KV
+                         end, ExtraEnv0),
+    Env = build_env(Arg, Scriptfilename, Pathinfo, ExtraEnv, SC),
     ?Debug("~p~n", [Env]),
     CGIPort = open_port({spawn, Exefilename},
                         [{env, Env},
@@ -802,8 +810,10 @@ call_fcgi(Role, Arg, Options) ->
                    [Role, fcgi_role_name(Role),
                     Options,
                     Arg]),
+            GlobalConf = get(gc),
             ServerConf = get(sc),
-            WorkerPid = fcgi_start_worker(Role, Arg, ServerConf, Options),
+            WorkerPid = fcgi_start_worker(Role, Arg, GlobalConf, ServerConf,
+              Options),
             handle_clidata(Arg, WorkerPid)
     end.
 
@@ -829,12 +839,12 @@ fcgi_worker_fail_if(true, WorkerState, Reason) ->
 fcgi_worker_fail_if(_Condition, _WorkerState, _Reason) ->
     ok.
 
-fcgi_start_worker(Role, Arg, ServerConf, Options) ->
+fcgi_start_worker(Role, Arg, GlobalConf, ServerConf, Options) ->
     proc_lib:spawn(?MODULE, fcgi_worker,
-                   [self(), Role, Arg, ServerConf, Options]).
+                   [self(), Role, Arg, GlobalConf, ServerConf, Options]).
 
 
-fcgi_worker(ParentPid, Role, Arg, ServerConf, Options) ->
+fcgi_worker(ParentPid, Role, Arg, GlobalConf, ServerConf, Options) ->
     {DefaultSvrHost, DefaultSvrPort} =
         case ServerConf#sconf.fcgi_app_server of
             undefined ->
@@ -857,9 +867,11 @@ fcgi_worker(ParentPid, Role, Arg, ServerConf, Options) ->
                             ?sc_fcgi_trace_protocol(ServerConf)),
     LogAppError = get_opt(log_app_error, Options,
                           ?sc_fcgi_log_app_error(ServerConf)),
+    TcpOptions = yaws:gconf_nslookup_pref(GlobalConf),
     AppServerSocket =
         fcgi_connect_to_application_server(PreliminaryWorkerState,
-                                           AppServerHost, AppServerPort),
+                                           AppServerHost, AppServerPort,
+                                           TcpOptions),
     ?Debug("Start FastCGI worker:~n"
            "  Role = ~p (~s)~n"
            "  AppServerHost = ~p~n"
@@ -910,9 +922,10 @@ fcgi_pass_through_client_data(WorkerState) ->
     end.
 
 
-fcgi_connect_to_application_server(WorkerState, Host, Port) ->
-    Options = [binary, {packet, 0}, {active, false}, {nodelay, true}],
-    case gen_tcp:connect(Host, Port, Options, ?FCGI_CONNECT_TIMEOUT_MSECS) of
+fcgi_connect_to_application_server(WorkerState, Host, Port, TcpOptions) ->
+    Options = [binary, {packet, 0}, {active, false}, {nodelay, true} |
+      TcpOptions],
+    case yaws:tcp_connect(Host, Port, Options, ?FCGI_CONNECT_TIMEOUT_MSECS) of
         {error, Reason} ->
             fcgi_worker_fail(WorkerState,
                              {"connect to application server failed", Reason});
@@ -1036,6 +1049,12 @@ fcgi_encode_record(WorkerState, Type, RequestId, NameValueList)
     fcgi_encode_record(WorkerState, Type, RequestId,
                        fcgi_encode_name_value_list(NameValueList));
 
+fcgi_encode_record(WorkerState, Type, RequestId, ContentData)
+  when is_binary(ContentData), size(ContentData) > 65535  ->
+    <<Bin:65535/binary, Rest/binary>> = ContentData,
+    [fcgi_encode_record(WorkerState, Type, RequestId, Bin),
+     fcgi_encode_record(WorkerState, Type, RequestId, Rest)];
+
 fcgi_encode_record(WorkerState, Type, RequestId, ContentData)
   when is_binary(ContentData) ->
     Version = 1,
@@ -1072,8 +1091,10 @@ fcgi_encode_name_value_list(_NameValueList = [{Name, Value} | Tail]) ->
 
 fcgi_encode_name_value(Name, _Value = undefined) ->
     fcgi_encode_name_value(Name, "");
-fcgi_encode_name_value(Name, Value) when is_list(Name) and is_list(Value) ->
-    NameSize = iolist_size(Name),
+fcgi_encode_name_value(Name0, Value0) ->
+    Name = unicode:characters_to_binary(Name0),
+    Value = unicode:characters_to_binary(Value0),
+    NameSize = byte_size(Name),
     %% If name size is < 128, encode it as one byte with the high bit clear.
     %% If the name size >= 128, encoded it as 4 bytes with the high bit set.
     NameSizeData = if
@@ -1083,15 +1104,14 @@ fcgi_encode_name_value(Name, Value) when is_list(Name) and is_list(Value) ->
                            <<(NameSize bor 16#80000000):32>>
                    end,
     %% Same encoding for the value size.
-    ValueSize = iolist_size(Value),
+    ValueSize = byte_size(Value),
     ValueSizeData = if
                         ValueSize < 128 ->
                             <<ValueSize:8>>;
                         true ->
                             <<(ValueSize bor 16#80000000):32>>
                     end,
-    list_to_binary([<<NameSizeData/binary,
-                      ValueSizeData/binary>>, Name, Value]).
+    list_to_binary([<<NameSizeData/binary, ValueSizeData/binary>>, Name, Value]).
 
 
 fcgi_header_loop(WorkerState) ->
@@ -1174,11 +1194,13 @@ fcgi_add_resp(WorkerState, OldData) ->
 
 fcgi_stream_data_loop(WorkerState) ->
     YawsWorkerPid = WorkerState#fcgi_worker_state.yaws_worker_pid,
-    case fcgi_get_output(WorkerState) of
+    case catch fcgi_get_output(WorkerState) of
         {data, Data} ->
             yaws_api:stream_chunk_deliver_blocking(YawsWorkerPid, Data),
             fcgi_stream_data_loop(WorkerState);
         {exit_status, _Status} ->
+            yaws_api:stream_chunk_end(YawsWorkerPid);
+        {'EXIT', _Reason} ->
             yaws_api:stream_chunk_end(YawsWorkerPid)
     end.
 
