@@ -42,8 +42,8 @@
 
 %% API
 -export([encode/2,
-	 decode/1,
-	 pad_len/1]).
+         decode/1,
+         pad_len/1]).
 
 
 -include_lib("kernel/include/file.hrl").
@@ -65,13 +65,16 @@
 encode(Req, []) ->
     encode_part(1, 1, 0, ?T_ABS_URI, <<"">>, <<"">>, <<?SOAP_URI>>, Req);
 encode(Req, As) ->
-	list_to_binary([encode_part(1, 0, 0, ?T_ABS_URI, <<"">>, <<"">>, <<?SOAP_URI>>, Req)|
-	encode_attachments(As)]).
+        list_to_binary([encode_part(1, 0, 0, ?T_ABS_URI,
+                                    <<"">>, <<"">>, <<?SOAP_URI>>, Req)|
+        encode_attachments(As)]).
 
 encode_attachments([{attachment, Id, Type, File}]) ->
-    [encode_part(0, 1, 0, ?T_ABS_URI, <<"">>, list_to_binary(Id), list_to_binary(Type), File)];
+    [encode_part(0, 1, 0, ?T_ABS_URI, <<"">>, list_to_binary(Id),
+                 list_to_binary(Type), File)];
 encode_attachments([{attachment, Id, Type, File} | As]) ->
-    [encode_part(0, 0, 0, ?T_ABS_URI, <<"">>, list_to_binary(Id), list_to_binary(Type), File)|
+    [encode_part(0, 0, 0, ?T_ABS_URI, <<"">>, list_to_binary(Id),
+                 list_to_binary(Type), File)|
      encode_attachments(As)].
 
 encode_part(MB, ME, CF, TypeT, Opts, ID, Type, Data) ->
@@ -96,10 +99,10 @@ encode_part(MB, ME, CF, TypeT, Opts, ID, Type, Data) ->
 
 pad_len(Len) ->
     case Len rem 4 of
-	0 ->
-	    0;
-	N ->
-	    4-N
+        0 ->
+            0;
+        N ->
+            4-N
     end.
 
 size_of(X) when is_list(X) ->
@@ -129,57 +132,57 @@ decode_recs(<<>>, Acc, _Chunks) ->
     Acc;
 decode_recs(Msg, Acc, Chunks) ->
     case {decode_rec(Msg), Chunks} of
-	%% A normal record
-	{{_ME=0, _CF=0, Opts, ID, Type, Chunk, More}, []} ->
-	    decode_recs(More, [{Opts, ID, Type, Chunk}|Acc], []);
+        %% A normal record
+        {{_ME=0, _CF=0, Opts, ID, Type, Chunk, More}, []} ->
+            decode_recs(More, [{Opts, ID, Type, Chunk}|Acc], []);
 
-	%% The last chunk of a block, but not the last record
-	{{_ME=0, _CF=0, Opts, ID, Type, Chunk, More}, Chunks} ->
-	    Cs = lists:reverse([{Opts, ID, Type, Chunk}|Chunks]),
-	    %% Only the first chunk has values for Opts, Id and Type
-	    {ROpts, RID, RType, _Chunk} = hd(Cs),
-	    Block = list_to_binary([Ch || {_Opts, _ID, _Type, Ch} <- Cs]),
-	    error_logger:info_report([?MODULE, decode_rec, merging,
-				      {id, ID},{type, Type},
-				      {chunks, length(Cs)}]),
-	    decode_recs(More, [{ROpts, RID, RType, Block}|Acc], []);
+        %% The last chunk of a block, but not the last record
+        {{_ME=0, _CF=0, Opts, ID, Type, Chunk, More}, Chunks} ->
+            Cs = lists:reverse([{Opts, ID, Type, Chunk}|Chunks]),
+            %% Only the first chunk has values for Opts, Id and Type
+            {ROpts, RID, RType, _Chunk} = hd(Cs),
+            Block = list_to_binary([Ch || {_Opts, _ID, _Type, Ch} <- Cs]),
+            error_logger:info_report([?MODULE, decode_rec, merging,
+                                      {id, ID},{type, Type},
+                                      {chunks, length(Cs)}]),
+            decode_recs(More, [{ROpts, RID, RType, Block}|Acc], []);
 
-	%% Last record, but not chunked
-	{{_ME=1, _CF=0, Opts, ID, Type, Data, _More}, []} ->
-	    lists:reverse([{Opts, ID, Type, Data}|Acc]);
+        %% Last record, but not chunked
+        {{_ME=1, _CF=0, Opts, ID, Type, Data, _More}, []} ->
+            lists:reverse([{Opts, ID, Type, Data}|Acc]);
 
-	%% Last record, and last chunk of block
-	{{_ME=1, _CF=0, Opts, ID, Type, Chunk, _More}, Chunks} ->
-	    Cs = lists:reverse([{Opts, ID, Type, Chunk}|Chunks]),
-	    {ROpts, RID, RType, _Chunk} = hd(Cs),
-	    Block = list_to_binary([Ch || {_Opts, _ID, _Type, Ch} <- Cs]),
-	    error_logger:info_report([?MODULE, decode_rec, merging,
-				      {id, ID},{type, Type},
-				      {chunks, length(Cs)}]),
-	    lists:reverse([{ROpts, RID, RType, Block}|Acc]);
+        %% Last record, and last chunk of block
+        {{_ME=1, _CF=0, Opts, ID, Type, Chunk, _More}, Chunks} ->
+            Cs = lists:reverse([{Opts, ID, Type, Chunk}|Chunks]),
+            {ROpts, RID, RType, _Chunk} = hd(Cs),
+            Block = list_to_binary([Ch || {_Opts, _ID, _Type, Ch} <- Cs]),
+            error_logger:info_report([?MODULE, decode_rec, merging,
+                                      {id, ID},{type, Type},
+                                      {chunks, length(Cs)}]),
+            lists:reverse([{ROpts, RID, RType, Block}|Acc]);
 
-	%% First or intermediate chunk, but not the last
-	{{_ME=0, _CF=1, Opts, ID, Type, Data, More}, Chunks} ->
-	    decode_recs(More, Acc, [{Opts, ID, Type, Data}|Chunks]);
+        %% First or intermediate chunk, but not the last
+        {{_ME=0, _CF=1, Opts, ID, Type, Data, More}, Chunks} ->
+            decode_recs(More, Acc, [{Opts, ID, Type, Data}|Chunks]);
 
-	%% Something wrong, ME=1 and CF=1
-	ErrorResult ->
-	        error_logger:error_report([?MODULE, decode_recs,
-					   {result, ErrorResult}])
+        %% Something wrong, ME=1 and CF=1
+        ErrorResult ->
+                error_logger:error_report([?MODULE, decode_recs,
+                                           {result, ErrorResult}])
     end.
 
 decode_rec(<<?VERSION1:5, _MB:1, ME:1, CF:1, _Type_T:4, _Res:4,
-	    Opt_Len:1/big-integer-unit:16,
-	    ID_Len:1/big-integer-unit:16,
-	    Type_Len:1/big-integer-unit:16,
-	    Data_Len:1/big-integer-unit:32,
-	    Rest/binary>>=Block) ->
+            Opt_Len:1/big-integer-unit:16,
+            ID_Len:1/big-integer-unit:16,
+            Type_Len:1/big-integer-unit:16,
+            Data_Len:1/big-integer-unit:32,
+            Rest/binary>>=Block) ->
     error_logger:info_report([?MODULE, decode_rec,
-			      {version,1},
-			      {mb,_MB},{me,ME},{cf,CF},{type_t,_Type_T},
-			      {res,_Res},{opt_len,Opt_Len},{id_len,ID_Len},
-			      {type_len,Type_Len},{date_len,Data_Len},
-			      {total_rec_size,byte_size(Block)}]),
+                              {version,1},
+                              {mb,_MB},{me,ME},{cf,CF},{type_t,_Type_T},
+                              {res,_Res},{opt_len,Opt_Len},{id_len,ID_Len},
+                              {type_len,Type_Len},{date_len,Data_Len},
+                              {total_rec_size,byte_size(Block)}]),
     Opt_pad = pad_len(Opt_Len),
     Id_pad = pad_len(ID_Len),
     Type_pad = pad_len(Type_Len),
@@ -187,33 +190,33 @@ decode_rec(<<?VERSION1:5, _MB:1, ME:1, CF:1, _Type_T:4, _Res:4,
     Header = Opt_Len+Opt_pad+ID_Len+Id_pad+Type_Len+Type_pad,
     DataTot = Data_Len+Data_pad,
     case byte_size(Rest) of
-	N when N >= Header+DataTot ->
-	    <<Opts:Opt_Len/binary-unit:8, _:Opt_pad/binary-unit:8,
-	     ID:ID_Len/binary-unit:8,     _:Id_pad/binary-unit:8,
-	     Type:Type_Len/binary-unit:8, _:Type_pad/binary-unit:8,
-	     Data:Data_Len/binary-unit:8, _:Data_pad/binary-unit:8,
-	     More/binary >> = Rest,
-	    {ME, CF, Opts, ID, Type, Data, More};
-	N when N > Header->
- 	    <<Opts:Opt_Len/binary-unit:8, _:Opt_pad/binary-unit:8,
- 	     ID:ID_Len/binary-unit:8,     _:Id_pad/binary-unit:8,
- 	     Type:Type_Len/binary-unit:8, _:Type_pad/binary-unit:8,
- 	     Data/binary>> = Rest,
-	    error_logger:error_report([?MODULE, decode_rec, short_record,
-				       {need, Header+DataTot},
-				       {has, N}]),
-	    {ME, CF, Opts, ID, Type, Data, <<>>};
-	N ->
-	    error_logger:error_report([?MODULE, decode_rec, short_record,
-				       {need, Header+DataTot},
-				       {has, N}]),
-	    {error, not_enough_data}
+        N when N >= Header+DataTot ->
+            <<Opts:Opt_Len/binary-unit:8, _:Opt_pad/binary-unit:8,
+             ID:ID_Len/binary-unit:8,     _:Id_pad/binary-unit:8,
+             Type:Type_Len/binary-unit:8, _:Type_pad/binary-unit:8,
+             Data:Data_Len/binary-unit:8, _:Data_pad/binary-unit:8,
+             More/binary >> = Rest,
+            {ME, CF, Opts, ID, Type, Data, More};
+        N when N > Header->
+            <<Opts:Opt_Len/binary-unit:8, _:Opt_pad/binary-unit:8,
+             ID:ID_Len/binary-unit:8,     _:Id_pad/binary-unit:8,
+             Type:Type_Len/binary-unit:8, _:Type_pad/binary-unit:8,
+             Data/binary>> = Rest,
+            error_logger:error_report([?MODULE, decode_rec, short_record,
+                                       {need, Header+DataTot},
+                                       {has, N}]),
+            {ME, CF, Opts, ID, Type, Data, <<>>};
+        N ->
+            error_logger:error_report([?MODULE, decode_rec, short_record,
+                                       {need, Header+DataTot},
+                                       {has, N}]),
+            {error, not_enough_data}
     end;
 
 decode_rec(Bin) ->
     error_logger:error_report([?MODULE, decode_rec, short_record,
-			       no_header,
-			       {has, byte_size(Bin)}]),
+                               no_header,
+                               {has, byte_size(Bin)}]),
     {error, no_header}.
 
 
