@@ -461,7 +461,7 @@ fmt_ip(HostName, _) ->
 
 fmtnow() ->
     {{Year, Month, Day}, {Hour, Min, Sec}} =
-        calendar:now_to_local_time(now()),
+        calendar:now_to_local_time(yaws:get_time_tuple()),
     ["[",fill_zero(Day,2),"/",yaws:month(Month),"/",integer_to_list(Year),":",
      fill_zero(Hour,2),":",fill_zero(Min,2),":",
      fill_zero(Sec,2)," ",zone(),"]"].
