@@ -95,9 +95,10 @@ not_found_body(Path, _GC, _SC) ->
 %% This function can only return an {ehtml, EH} or an {html, HTML}
 %% value, no status codes, no headers etc.
 crashmsg(_Arg, _SC, L) ->
+    error_logger:format("~s", [L]),
     {ehtml,
      [{h2, [], "Internal error, yaws code crashed"},
       {br},
       {hr},
-      {pre, [], L},
+      {pre, [], yaws_api:htmlize(L)},
       {hr}]}.
