@@ -574,24 +574,19 @@ connect(URL) ->
     end.
 
 do_connect(URL) ->
-    InetType = if
-                   is_tuple(URL#url.host), size(URL#url.host) == 8 -> [inet6];
-                   true -> []
-               end,
     Opts = [
             binary,
             {packet,    raw},
             {active,    false},
-            {recbuf,    8192},
             {reuseaddr, true}
-           ] ++ InetType,
+           ],
     case URL#url.scheme of
         http  ->
             Port = case URL#url.port of
                        undefined -> 80;
                        P         -> P
                    end,
-            case gen_tcp:connect(URL#url.host, Port, Opts) of
+            case yaws:tcp_connect(URL#url.host, Port, Opts) of
                 {ok, S} -> {ok, S, nossl};
                 Err     -> Err
             end;
@@ -600,7 +595,7 @@ do_connect(URL) ->
                        undefined -> 443;
                        P         -> P
                    end,
-            case ssl:connect(URL#url.host, Port, Opts) of
+            case yaws:ssl_connect(URL#url.host, Port, Opts) of
                 {ok, S} -> {ok, S, ssl};
                 Err     -> Err
             end;
