@@ -1,8 +1,8 @@
 -module(flushtest).
 -export([out/1]).
 
--include("../../include/yaws.hrl").
--include("../../include/yaws_api.hrl").
+-include("yaws.hrl").
+-include("yaws_api.hrl").
 
 out(_Arg) ->
     [{status, 200}, {html, "break"}, flush, break].
