@@ -1,8 +1,9 @@
 -module(app_test).
--include("../include/tftest.hrl").
--include_lib("ibrowse/include/ibrowse.hrl").
 -compile(export_all).
 
+-include_lib("ibrowse/include/ibrowse.hrl").
+-include("tftest.hrl").
+
 
 %% Way to invoke just one test
 start([F]) ->
@@ -63,7 +64,7 @@ deflate_enabled() ->
     ?line "Accept-Encoding" = proplists:get_value("Vary", Hdrs1),
     ?line true = is_binary(zlib:gunzip(Body1)),
 
-    %% Partial content is not compressed for small (and catched) files
+    %% Partial content is not compressed for small (and cached) files
     Uri2 = "http://localhost:8001/1000.txt",
     ?line {ok, "206", Hdrs2, _} =
         ibrowse:send_req(Uri2, [{"Accept-Encoding", "gzip, deflate"},
@@ -210,7 +211,7 @@ deflate_mime_types() ->
 deflate_compress_size() ->
     io:format("  deflate_compress_size\n", []),
 
-    %% Static content (cached)
+    %% Small static content (cached)
     Uri1 = "http://localhost:8005/1000.txt",
     ?line {ok, "200", Hdrs1, _} =
         ibrowse:send_req(Uri1, [{"Accept-Encoding", "gzip, deflate"}], get),
@@ -222,17 +223,29 @@ deflate_compress_size() ->
     ?line "gzip" = proplists:get_value("Content-Encoding", Hdrs2),
     ?line true = is_binary(zlib:gunzip(Body2)),
 
-    %% Dynamic content
-    Uri3 = "http://localhost:8005/smalltest",
-    ?line {ok, "200", Hdrs3, _} =
+    %% Large static content (cached)
+    Uri3 = "http://localhost:8005/10000.txt",
+    ?line {ok, "200", Hdrs3, Body3} =
         ibrowse:send_req(Uri3, [{"Accept-Encoding", "gzip, deflate"}], get),
-    ?line undefined = proplists:get_value("Content-Encoding", Hdrs3),
+    ?line "gzip" = proplists:get_value("Content-Encoding", Hdrs3),
+    ?line true = is_binary(zlib:gunzip(Body3)),
 
-    Uri4 = "http://localhost:8005/bigtest",
-    ?line {ok, "200", Hdrs4, Body4} =
+    Uri4 = "http://localhost:8008/10000.txt",
+    ?line {ok, "200", Hdrs4, _} =
         ibrowse:send_req(Uri4, [{"Accept-Encoding", "gzip, deflate"}], get),
-    ?line "gzip" = proplists:get_value("Content-Encoding", Hdrs4),
-    ?line true = is_binary(zlib:gunzip(Body4)),
+    ?line undefined = proplists:get_value("Content-Encoding", Hdrs4),
+
+    %% Dynamic content
+    Uri5 = "http://localhost:8005/smalltest",
+    ?line {ok, "200", Hdrs5, _} =
+        ibrowse:send_req(Uri5, [{"Accept-Encoding", "gzip, deflate"}], get),
+    ?line undefined = proplists:get_value("Content-Encoding", Hdrs5),
+
+    Uri6 = "http://localhost:8005/bigtest",
+    ?line {ok, "200", Hdrs6, Body6} =
+        ibrowse:send_req(Uri6, [{"Accept-Encoding", "gzip, deflate"}], get),
+    ?line "gzip" = proplists:get_value("Content-Encoding", Hdrs6),
+    ?line true = is_binary(zlib:gunzip(Body6)),
     ok.
 
 
@@ -310,4 +323,15 @@ deflate_otheroptions() ->
     ?line "chunked" = proplists:get_value("Transfer-Encoding", Hdrs5),
     ?line undefined = proplists:get_value("Content-Length", Hdrs5),
     ?line true = is_binary(zlib:gunzip(Body5)),
+    ok,
+
+    %% Multiple Accept-Encoding headers
+    %% This test reuses Uri1 and Body1. It sends two Accept-Encoding
+    %% headers to make sure Yaws handles them correctly. The reply
+    %% should be the same as for the static content test at the beginning
+    %% of this test function.
+    ?line {ok, "200", Hdrs6, Body1} =
+        ibrowse:send_req(Uri1, [{"Accept-Encoding", "identity;q=0.5"},
+                                {"Accept-Encoding", "gzip, deflate"}], get),
+    ?line "gzip" = proplists:get_value("Content-Encoding", Hdrs6),
     ok.
